import ReactQuill, { Quill } from 'react-quill'
import 'react-quill/dist/quill.snow.css'
import { useRef, useState } from 'react'
import { TOPS, TProofreadItem } from './types'
import { test_content_1 } from './convert'

const ProofreadList: TProofreadItem[] = [
    {
        id: 17009,
        uid: 3,
        fid: 290,
        key: "5aa5de13391f8038c55ab2174f13bb84",
        text: "虽然我没有相关的教学经验，但我觉得这是一个不错锻炼自己，并且能钱。",
        newText:
            "虽然我没有相关的教学经验，但我觉得这是一个锻炼自己，并且能赚钱的工作。",
        correctedContent: {
            tag: "d",
            origin: "打发",
            text: "",
            start: 21,
            end: 23,
        },
        action: "delete",
        checkTime: 1697186287,
        offset: 35,
        isAccept: 2,
        type: 2,
        ctime: 1697186287,
        utime: 1697199789,
        dtime: 0,
    },
    {
        id: 17010,
        uid: 3,
        fid: 290,
        key: "5aa5de13391f8038c55ab2174f13bb84",
        text: "虽然我没有相关的教学经验，但我觉得这是一个不错锻炼自己，并且能钱。",
        newText:
            "虽然我没有相关的教学经验，但我觉得这是一个锻炼自己，并且能赚钱的工作。",
        correctedContent: {
            tag: "i",
            origin: "",
            text: "赚",
            start: 29,
            end: 29,
        },
        action: "insert",
        checkTime: 1697186287,
        offset: 35,
        isAccept: 0,
        type: 2,
        ctime: 1697186287,
        utime: 1697199789,
        dtime: 0,
    },
    {
        id: 17011,
        uid: 3,
        fid: 290,
        key: "5aa5de13391f8038c55ab2174f13bb84",
        text: "虽然我没有相关的教学经验，但我觉得这是一个不错锻炼自己，并且能钱。",
        newText:
            "虽然我没有相关的教学经验，但我觉得这是一个锻炼自己，并且能赚钱的工作。",
        correctedContent: {
            tag: "i",
            origin: "",
            text: "的工作",
            start: 30,
            end: 30,
        },
        action: "insert",
        checkTime: 1697186287,
        offset: 35,
        isAccept: 2,
        type: 2,
        ctime: 1697186287,
        utime: 1697199789,
        dtime: 0,
    },
    {
        id: 17012,
        uid: 3,
        fid: 290,
        key: "dbe3ddd3dae993521b00f39c01744695",
        text: "\n然而，有一天晚上，我在酒吧喝了太多酒，失去失去了意识。",
        newText: "然而，有一天晚上，我在酒吧喝了太多酒，失去了意识。",
        correctedContent: {
            tag: "d",
            origin: "失去",
            text: "",
            start: 22,
            end: 24,
        },
        action: "delete",
        checkTime: 1697186287,
        offset: 69,
        isAccept: 0,
        type: 2,
        ctime: 1697186287,
        utime: 1697199789,
        dtime: 0,
    },
    {
        id: 17013,
        uid: 3,
        fid: 290,
        key: "895f18874c0b473558d49409872c78ea",
        text: "在酒经的作用下，我变得迷迷糊糊，不知道自己在做什么。",
        newText: "在酒精的作用下，我变得迷迷糊糊，不知道自己在做什么。",
        correctedContent: {
            tag: "r",
            origin: "经",
            text: "精",
            start: 2,
            end: 3,
        },
        action: "replace",
        checkTime: 1697186287,
        offset: 97,
        isAccept: 0,
        type: 2,
        ctime: 1697186287,
        utime: 1697199789,
        dtime: 0,
    },
]
// 将数组转换成对象
const proofreadData: Map<number, TProofreadItem> = new Map();
for (let i = 0; i < ProofreadList.length; i++) {
    proofreadData.set(ProofreadList[i].id, ProofreadList[i])
}

const content: {
    ops: Array<TOPS>
} = {
    ops: test_content_1
}

const BasicEditor = () => {
    const quill = useRef<ReactQuill>(null)

    const [data, setData] = useState<{
        content: string;
        text?: string;
        ops: any
    }>({
        content: '',
        text: '',
        ops: []
    })

    const setContent = () => {
        const editor = quill.current?.editor;
        if (!editor) return;
        editor.setContents(content as any)
    }

    const getContent = () => {
        const editor = quill.current?.editor;
        if (!editor) return;
        const content = editor.getContents()
        content.ops?.forEach((it) => {
            // const proofread = ;
            if (it.attributes?.delete && it.attributes.delete.id) {
                const proofread = proofreadData.get(Number(it.attributes.delete.id));
                if (proofread) {
                    console.log('delete proofread data', proofread)
                }
            }
        })
        console.log(
            content
        )
    }
    let prevSelectNode: HTMLSpanElement | null = null;
    const handleEditorClick = (e: React.MouseEvent<HTMLDivElement>) => {
        const node = e.target as HTMLSpanElement;
        if (!node || !node.classList.contains('proofread-item') || !node.getAttribute('data-proofread-id') || prevSelectNode == node) return;
        prevSelectNode?.classList.remove('selected')
        e.preventDefault()
        e.stopPropagation()
        node.classList.add('selected')
        prevSelectNode = node;
    }

    const setCursor = () => {
        if(!quill.current) return;
        const node = document.querySelector('.proofread-item-170')
        if(!node) return;
        
        node.setAttribute('data-result', '~~~');
        node.innerHTML = '<del></del>';
        // node.appendChild(document.createElement('del'));
        const editor = quill.current.editor!;
        const b = Quill.find(node);
        var i = editor.getIndex(b);
        // console.log(i)
        editor?.setSelection(i, 0, 'silent')
    }

    return (
        <div className='container'>
            <div className='editor-container' style={{ width: '100%', marginTop: 10 }} onClick={handleEditorClick}>
                <ReactQuill
                    ref={quill}
                    theme='snow'
                    modules={{ toolbar: false }}
                    onChangeSelection={(selection) => {
                        if (!selection) return;
                        // handleEditorSelectionChange(selection.index)
                        // console.log(
                        //   editor.getBounds(selection!.index)
                        // )
                    }}
                    onChange={(content, _delta, source) => {
                        // console.log('source', source, _delta)
                        const ops = quill.current!.editor!.getContents().ops
                        let originText = data.text || ''
                        let text = quill.current?.editor?.getText() || '';
                        setData({
                            content,
                            text,
                            ops
                        })
                        // if (source === 'user') {
                        //     handleEditorTextChange(text, originText)
                        // }
                    }} />
            </div>
            {/* <button onClick={origin}>origin</button>
            <button onClick={handleProcess}>process</button>
            <button onClick={testContentEdit}>testContentEdit</button> */}
            <div style={{ margin: '10px 0' }}>
                <button onClick={getContent}>test</button>
                <button onClick={setContent}>setContent</button>
                <button onClick={setCursor}>setCursor</button>
            </div>
            <div className="code-wrapper">
                <div>{JSON.stringify(data, null, '\t')}</div>
            </div>
        </div>
    )
}

export default BasicEditor