import ReactQuill from 'react-quill'
import 'react-quill/dist/quill.snow.css'
import { useEffect, useRef, useState } from 'react'
import { TOPS } from '../types'
import { Button, Dropdown, MenuProps, Space } from 'antd'

type TError = {
    id: number;
    offset: number;
    length: number;
}
type ErrorPosition = {
    index: number;
    error?: TError;
    left: number;
    top: number;
    width: number;
    height: number;
    offset: number;
    length: number;
}
const content: {
    ops: Array<TOPS>
} = {
    ops: [
        {
            insert: "hello "
        },
        {
            attributes: {
                bold: true
            },
            insert: "world"
        },
    ]
}
content.ops = [
    {
        insert: "该摄"
    },
    {
        attributes: {
            insert: {
                origin: '',
                text: '像'
            }
        },
        insert: '像'
    },
    {
        attributes: {
            bold: true,
            italic: true
        },
        insert: "头头"
    },
    {
        insert: "可以在单"
    },
    {
        attributes: {
            bold: true,
            replace: {
                origin: '次爆',
                text: '次爆爆'
            }
        },
        insert: "次爆"
    },
    {
        attributes: {
            italic: true
        },
        insert: "光中以每秒"
    },
    {
        "insert": " 480 万帧的速度采集动态的事件\n的图像。\n\n采集快速运动过程（如落下的水滴或分子相互作用）的 青晰 图像，我们需要使用昂贵的超快摄像头（每秒可采集数百万张图像）。据国外媒报道，加拿大国立科学研究院（INRS）宣布开发出新摄像头，能够以极低的成本实现超快速成像，并"
    },
    {
        attributes: {
            bold: true
        },
        insert: "适合"
    },
    {
        attributes: {
            italic: true,
            bold: true
        },
        insert: "多种(many)"
    },
    {
        insert: "应用，例如实时监测药物输送或用于自动驾驶的高速激光雷达系统。\n"
    }
]



/**
 * 错误列表
 */
const ResultErrorList = [
    { offset: 4, length: 1, id: 1, },
    { offset: 10, length: 2, id: 2 },
    { offset: 28, length: 1, id: 3 },
    { offset: 30, length: 6, id: 4 },
    { offset: 124, length: 7, id: 5 },
]
function Editor() {
    const quill = useRef<ReactQuill>(null)
    useEffect(() => {
        const editor = quill.current?.editor;
        if (!editor) return;
        editor.setContents(content as any)
    }, [quill])
    const [data, setData] = useState<{
        content: string;
        text?: string;
        ops: any
    }>({
        content: '',
        text: '',
        ops: []
    })
    // const [state, setState] = useState<{
    //   bottom?: number;
    //   height?: number;
    //   left?: number;
    //   right?: number;
    //   top?: number;
    //   width?: number;
    // }>({})
    const offset = 10, length = 2
    const [editor, setEditor] = useState<{
        div?: HTMLDivElement,
        info: {
            height: number;
            width: number;
            x: number;
            y: number;
            bottom: number;
            left: number;
            right: number;
            top: number;
        }
    }>({
        info: {
            width: 0,
            height: 0,
            left: 0,
            top: 0,
            right: 0,
            bottom: 0,
            x: 0,
            y: 0
        }
    });
    const getAllChildNodes = (node: ChildNode) => {
        const childNodes = node.childNodes;
        if (childNodes.length > 0) {
            const _: any = [];
            for (let i = 0; i < childNodes.length; i++) {
                _.push(getAllChildNodes(childNodes[i]))
            }
            return _
        }
        return node
    }
    // 根据位置获得节点索引
    const getNodeIndexByOffsetAndLength = (lengthList: number[], offset: number, length: number) => {
        const posList: number[][] = []
        // 开始范围位置
        let startPos = 0
        // 结束范围位置
        let endPos = 0;
        for (let i = 0; i < lengthList.length; i++) {
            if (lengthList[i] === 0) continue; // 跳过空
            endPos = startPos + lengthList[i];
            //1.完全在该长度范围内容
            if (offset >= startPos && (offset + length) < endPos) {
                posList.push([i, offset - startPos, length])
                return posList
            }
            //2. 开始在范围内，结束没有在范围内
            if (offset >= startPos && offset < endPos && (offset + length) >= endPos) {
                const curLen = endPos - offset
                posList.push([i, offset - startPos, curLen])
                // 剩余长度减去已经处理的值
                length = length - curLen;
                if (length === 0) return posList;
                // 开始位置为当前位置结束 + 1
                offset = endPos;
            }
            startPos = endPos;
        }
        return posList
    }

    const calcRectInfo = (rect: DOMRect) => {
        return {
            left: rect.left - editor.info.left,
            top: rect.top - editor.info.top,
            width: rect.width,
            height: rect.height,
        }
    }
    const getBounds = (textNodes: Node[], pos: number[][], error: TError) => {
        const bounds: ErrorPosition[] = []
        for (let i = 0; i < pos.length; i++) {
            const [index, offset, length] = pos[i]
            const node = textNodes[index]
            // start range
            let rangeStart: Range | null = document.createRange();
            rangeStart.setStart(node, 0);
            rangeStart.setEnd(node, 0);
            // start height
            const startHeight = rangeStart.getBoundingClientRect().height;
            rangeStart.detach()
            rangeStart = null;
            // content range
            let range: Range | null = document.createRange()
            range.setStart(node, offset)
            range.setEnd(node, offset + length)
            const rect = calcRectInfo(range.getBoundingClientRect())

            range.detach();
            range = null;
            if (Math.floor(rect.height) == Math.floor(startHeight)) { // start equal end so 在一行了啦
                bounds.push({
                    index,
                    error,
                    offset,
                    length,
                    ...rect
                })
            } else {
                // not in same line, calculate each char in string  
                const highlights: any[] = [];
                let start = 0, count = 1;
                while (count <= length) { // 
                    let strRange: Range | null = document.createRange()
                    strRange.setStart(node, offset + start)
                    strRange.setEnd(node, offset + count)
                    const rect = calcRectInfo(strRange.getBoundingClientRect());
                    strRange.detach();
                    strRange = null;
                    if (Math.floor(rect.height) == Math.floor(startHeight)) { // in first line
                        if (count !== 1) highlights.pop();
                        count++;
                    } else {
                        if (start == count - 1) {
                            highlights.pop();
                            start = count;
                            count -= 1;
                        } else {
                            start = count - 1;
                        }
                    }
                    highlights.push({
                        index: start,
                        error,
                        offset: offset + start,
                        length: count,
                        ...rect
                    })
                }
                bounds.push(...highlights)
            }
        }
        return bounds
    }

    const [errPos, setErrPos] = useState<ErrorPosition[]>([])
    const getAllTextNodes = () => {
        const qcs = editor.div!.childNodes;
        const qcList = [];
        for (let i = 0; i < qcs.length; i++) {
            qcList.push(getAllChildNodes(qcs[i]))
        }
        return qcList.flat(10) as Node[];
    }
    const origin = () => {
        const nodeList = getAllTextNodes();
        const lenList: number[] = []
        nodeList.forEach(node => {
            if (node.nodeName === '#text') {
                lenList.push((node as Text).length)
            } else {
                lenList.push(0)
            }
        })
        const bounds: ErrorPosition[][] = [];
        ResultErrorList.forEach(err => {
            if (ErrorData.find(s => s.id == err.id)?.processed) return;
            const posList = getNodeIndexByOffsetAndLength(
                lenList,
                err.offset,
                err.length
            )
            // posList.forEach(p => {
            //   const [index, offset, length] = p;
            //   console.log(nodeList[index], p, nodeList[index].textContent?.slice(offset, offset + length))
            // })
            bounds.push(getBounds(nodeList, posList, err))
        })
        // const bounds = getBounds(nodeList, lenList)
        setErrPos(bounds.flat(1))
    }

    useEffect(() => {
        const div = document.querySelector<HTMLDivElement>('.ql-editor')!
        setEditor({
            div,
            info: div.getBoundingClientRect(),
        })
    }, [quill])
    const [selectId, setSelectId] = useState<number>(-1)
    const handleEditorSelectionChange = (index: number) => {
        const editor = quill.current?.editor!;
        const info = editor.getBounds(index)
        console.log('editor change info', info)
        let id = -1;
        errPos.forEach(err => {
            // 判断是否在范围内
            if (
                info.left >= err.left && info.left <= (err.left + err.width)
                && info.top >= err.top && info.top <= (err.top + err.height)
            ) {
                id = err.error?.id || -1
            }
        })
        setSelectId(id)
    }
    const handleEditorTextChange = (newText: string, originText: string) => {
        const range = window.getSelection()?.getRangeAt(0)
        if (!range) return;
        originText = originText.replace(/&nbsp;/gi, " ");
        newText = newText.replace(/&nbsp;/gi, " ");
        const count = newText.length - originText.length;
        const nodeList = getAllTextNodes();
        let startOffset = 0;
        // 计算偏移量
        for (let node of nodeList) {
            if (range.startContainer == node) {
                // 找到该节点就不计算后续的了
                startOffset += range.startOffset;
                break;
            }
            if (node.nodeName === '#text') startOffset += (node as Text).length
        }

        console.log(JSON.stringify(ResultErrorList))
        // 更新位置
        for (let i = 0; i < ResultErrorList.length; i++) {
            // 判断要更新的位置是否在修改位置后面
            if (ResultErrorList[i].offset >= startOffset - count) {
                ResultErrorList[i].offset += count;
            }
        }
        console.log(JSON.stringify(ResultErrorList))
        origin()
    }

    const ErrorData = [
        { id: 1, tag: 'd', origin: '头', text: '', processed: false },
        { id: 2, tag: 'd', origin: '爆光', text: '曝光', processed: false },
        { id: 3, tag: 'r', origin: '动', text: '洞', processed: false },
        { id: 4, tag: 'd', origin: '动', text: '洞', processed: false },
    ]
    const handleProcess = () => {
        console.log('selectId', selectId)
        const err = ErrorData.find(item => item.id === selectId)
        if (!err) return;
        const nodeList = getAllTextNodes();
        const processList = errPos.filter(s => s.error?.id == selectId)
        err.processed = true;
        processList.forEach(it => {
            const node = nodeList[it.index] as Text;
            const content = node.textContent!
            // 
            // console.log(it, nodeList[it.index])
            if (err.tag == 'd') {
                // 删除
                node.textContent = content.slice(0, it.offset) + content.slice(it.offset + it.length)
            } else if (err.tag == 'r') { // 替换
                console.log(node.textContent!.slice(0, it.offset), node.textContent!.slice(it.offset + it.length))
                node.textContent = node.textContent!.slice(0, it.offset) + err.text + node.textContent!.slice(it.offset + it.length)
            }
        })
        // origin();
    }
    const test = () => {
        const editor = quill.current?.editor!;
        // editor.insertText(pos[0], 'hello world')
        // const bounds = editor.getBounds(pos[0], pos[1])
        // console.log(bounds)
        // setState(bounds)
        // editor.setSelection(pos[0], pos[1])
        editor.deleteText(offset, length)
        editor.insertText(offset, '爆光', {
            replace: {
                origin: '爆光',
                text: '曝光'
            }
        })
    }

    const testContentEdit = () => {
        const spans = document.querySelectorAll('.proofread-replace-item')
        Array.from(spans).forEach((node) => {
            console.log(node.textContent, node)
            const replacedText = node.getAttribute('data-text')
            if (replacedText) {
                //(node as HTMLElement).innerText = replacedText
                node.textContent = replacedText
            }
        })
        console.log(quill.current!.editor!.getText())
        setTimeout(() => {
            console.log(quill.current!.editor!.getText())
        }, 500);
    }
    const items: MenuProps['items'] = [
        {
            label: <a href="https://www.antgroup.com">1st menu item</a>,
            key: '0',
        },
        {
            label: <a href="https://www.aliyun.com">2nd menu item</a>,
            key: '1',
        },
        {
            type: 'divider',
        },
        {
            label: '3rd menu item',
            key: '3',
        },
    ];

    return (
        <div>
            <Space>
                <Button type='primary' onClick={origin}>origin</Button>
                <Button onClick={handleProcess}>process</Button>
                <Button onClick={test}>test</Button>
                <Button onClick={testContentEdit}>testContentEdit</Button>
            </Space>
            {/* <p>该摄像<strong><em>头头</em></strong>可以在单<i>次爆</i></p> */}
            <div className='editor-container' style={{ width: '100%', marginTop: 10 }}>
                <ReactQuill
                    ref={quill}
                    theme='snow'
                    modules={{ toolbar: false }}
                    onChangeSelection={(selection) => {
                        if (!selection) return;
                        handleEditorSelectionChange(selection.index)
                        // console.log(
                        //   editor.getBounds(selection!.index)
                        // )
                    }}
                    onChange={(content, _delta, source) => {
                        // console.log('source', source, _delta)
                        const ops = quill.current!.editor!.getContents().ops
                        let originText = data.text || ''
                        let text = quill.current?.editor?.getText() || '';
                        setData({
                            content,
                            text,
                            ops
                        })
                        if (source === 'user') {
                            handleEditorTextChange(text, originText)
                        }
                    }} />
                <div className="hilight">
                    {errPos.map((err, errIndex) => <Dropdown open={err.error && err.error?.id == selectId ? true : false} key={errIndex} menu={{ items }}>
                        <div data-error-id={err.error?.id} className={`hilight-item ${err.error?.id == selectId ? 'hilight-item-selected' : ''}`} style={{
                            left: err.left || 0,
                            top: err.top || 0,
                            width: err.width || 0,
                            height: err.height || 0,
                        }}></div>
                    </Dropdown>)}
                </div>
            </div>
            {/* <div className="code-wrapper">
                <pre>{JSON.stringify(data, null, '  ')}</pre>
            </div> */}
        </div>
    )
}

export default Editor
