import { useEffect, useRef } from 'react';
import * as THREE from 'three';
import { GLTFLoader } from 'three/addons/loaders/GLTFLoader.js';
import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls.js'

export type GLTFViewerProp = {
    gltfUrl: string;
    scale?: number;
    position?: [number, number, number];
    rotation?: [number, number, number];
    onLoad?: () => void;
    onError?: () => void;
    autoPlay?: boolean;
    width?: number;
    height?: number;
}
const GLTFViewer: React.FC<GLTFViewerProp> = (props) => {
    const { gltfUrl, scale = 1, position = [0, 0, 0], rotation = [0, 0, 0], width = 500, height = 400, autoPlay = true } = props;
    const viewer = useRef<HTMLDivElement>(null);

    const scence = new THREE.Scene();
    const camera = new THREE.PerspectiveCamera(75, width / height, 0.1, 1000);
    const renderer = new THREE.WebGLRenderer({ antialias: true, alpha: true });
    const controls = new OrbitControls(camera, renderer.domElement);
    const loader = new GLTFLoader();

    //光源
    const ambient = new THREE.AmbientLight(0xffffff, 0.4);
    scence.add(ambient);
    const directionalLight = new THREE.DirectionalLight(0xffffff, 0.8);//光源，color:灯光颜色，intensity:光照强度
    directionalLight.position.set(400, 200, 300);
    scence.add(directionalLight);
    //设置相机位置
    camera.position.set(-10, 100, 10);
    //设置相机方向
    camera.lookAt(0, 0, 0);
    //辅助坐标轴
    // const axesHelper = new THREE.AxesHelper(200);//参数200标示坐标系大小，可以根据场景大小去设置
    // scence.add(axesHelper);
    scence.background = new THREE.Color(0xeaeaea);
    // 
    renderer.setPixelRatio(window.devicePixelRatio);
    renderer.setSize(width, height);

    const render = ()=>{
        renderer.render(scence, camera);
        controls.update();
        requestAnimationFrame(render);
    }
    useEffect(() => {
        if (viewer.current) {
            viewer.current.appendChild(renderer.domElement);
            // renderer.render(scence, camera);
            loader.load(gltfUrl, (gltf) => {
                // console.log(gltf)
                gltf.scene.scale.set(5, 5, 5);//设置模型大小
                scence.add(gltf.scene);
                render();
            });
        }
    }, [viewer])
    

    return (<>
        <h1>{gltfUrl}</h1>
        <div ref={viewer} style={{ width, height }}></div>
    </>)

}
export default GLTFViewer;

// import { useGLTF } from '@react-three/drei'