import { Command, Extension } from '@tiptap/core';
import { Transaction } from '@tiptap/pm/state';

export interface IndentOptions {
    types: string[];
    minLevel: number;
    maxLevel: number;
}

declare module '@tiptap/core' {
    interface Commands {
        indent: {
            indent: () => Command;
            outdent: () => Command;
        };
    }
}
// 设置节点缩进
function setNodeIndent(tr: Transaction, delta: number) {
    if (!tr.doc || !tr.selection) return tr;

    // if(node.type.name === 'paragraph'){
    //     node.attrs.indent = node.attrs.indent + delta;
    // }
    return tr;
}

export const Indent = Extension.create<IndentOptions>({
    name: 'indent',

    defaultOptions: {
        types: ['paragraph'],
        minLevel: 1,
        maxLevel: 8,
    },
    addGlobalAttributes() {
        return [
            {
                types: this.options.types,
                attributes: {
                    indent: {
                        default: 0,
                        parseHTML: element => {
                            const { level } = element.dataset;
                            return level ? Number(level) : 0;
                        },
                        renderHTML: attributes => {
                            return {
                                'data-indent': attributes.indent,
                            };
                        },
                    },
                },
            },
        ];
    },
    addCommands() {
        const applyIndent = (delta: number) => ({ tr, state, dispatch }) => {
            const { selection } = state;
            tr = tr.setSelection(selection);
            tr = setNodeIndent(tr, delta)
            if (tr.docChanged) {
                // eslint-disable-next-line no-unused-expressions
                dispatch && dispatch(tr)
                return true
            }
            return false
        }
        return {
            indent: () => applyIndent(1),

            /**
             * Outdent a paragraph.
             */
            outdent: () => applyIndent(-1)
        }
    },
    addKeyboardShortcuts() {
        return {
            Tab: () => {
                this.editor.commands.indent();
            },
            'Shift-Tab': () => {
                this.editor.commands.outdent();
            }
        }
    },
});