import { createFromIconfontCN } from "@ant-design/icons";
import { Editor } from "@tiptap/react"
import { Popover } from "antd";
import React from "react"

type TiptapMenuBarProps = {
    editor: Editor | null
}

const IconFont = createFromIconfontCN({
    scriptUrl: 'https://at.alicdn.com/t/c/font_4382753_t9zvof0xm5.js',
});

export const TiptapMenuBar: React.FC<TiptapMenuBarProps> = (props) => {
    const { editor } = props
    const buttons = [
        { click: () => editor?.chain().focus().toggleBold().run(), disabled: false, icon: 'bold', key: 'bold' },
        { click: () => editor?.chain().focus().toggleItalic().run(), disabled: false, icon: 'italic', key: 'italic' },
        { click: () => editor?.chain().focus().toggleStrike().run(), disabled: false, icon: 'strikethrough', key: 'strike', tips: '删除' },
        { click: () => editor?.chain().focus().toggleCode().run(), disabled: false, icon: 'code', key: 'code' },
        { click: () => editor?.chain().focus().setParagraph().run(), disabled: false, icon: 'paragraph', key: 'paragraph' },
        { click: () => editor?.chain().focus().toggleBulletList().run(), disabled: false, icon: 'unorderedlist', key: 'bulletList' },
        { click: () => editor?.chain().focus().toggleOrderedList().run(), disabled: false, icon: 'orderedlist', key: 'orderedList' },
        { click: () => editor?.chain().focus().toggleBlockquote().run(), disabled: false, icon: 'quote', key: 'blockquote',tips:'引用' },
        { click: () => editor?.chain().focus().toggleBlockquote().run(), disabled: false, icon: 'quote', key: 'blockquote',tips:'引用' },
        { click: () => editor?.chain().focus().toggleBlockquote().run(), disabled: false, icon: 'quote', key: 'blockquote',tips:'引用' },
    ]
    return !editor ? null : (
        <div className="tiptap-menus">
            {buttons.map(btn => <button key={btn.key}
                onClick={btn.click}
                disabled={btn.disabled}
                className={editor.isActive(btn.key) ? 'is-active' : ''}
            >
                <Popover content={btn.tips || btn.key}><IconFont type={`icon-${btn.icon}`} /></Popover>
            </button>)}

            {[1, 2, 3, 4].map((level: any) => <button
                onClick={() => editor.chain().focus().toggleHeading({ level }).run()}
                className={editor.isActive('heading', { level }) ? 'is-active' : ''}
            >
                <Popover content={`标题${level}`}><IconFont type={`icon-h${level}`} /></Popover>
            </button>)}
            <button onClick={() => editor.chain().focus().setHorizontalRule().run()}>
                horizontal rule
            </button>
            <button onClick={() => editor.chain().focus().setHardBreak().run()}>
                hard break
            </button>
            <button
                onClick={() => editor.chain().focus().undo().run()}
                disabled={
                    !editor.can()
                        .chain()
                        .focus()
                        .undo()
                        .run()
                }
            >
                undo
            </button>
            <button
                onClick={() => editor.chain().focus().redo().run()}
                disabled={
                    !editor.can()
                        .chain()
                        .focus()
                        .redo()
                        .run()
                }
            >
                redo
            </button>
            <button
                onClick={() => editor.chain().focus().setColor('#958DF1').run()}
                className={editor.isActive('textStyle', { color: '#958DF1' }) ? 'is-active' : ''}
            >
                purple
            </button>
        </div>
    )
}