export type int = number;
export type bool = boolean;

// 0默认 1忽略 2采纳 3未采纳
export enum ProofreadStateEnum {
    Default,
    Ignore,
    Accept,
    NotAdopted,

}

//校验内容
export type TCorrectedContent = {
    /**
     * 'd' | 'i' | 'r'
     */
    tag: 'd' | 'i' | 'r' | string;
    //原内容
    origin: string;
    //替换内容
    text: string;
    //开始位置
    start: int;
    //结束位置
    end: int;
}
export enum ProofreadTypeEnum {
    unknown,
    blackWord,
    /**
     * 字错
     */
    characterError,
    /**
     * 词错
     */
    wordError,
    /**
     * 数字错
     */
    numberError,
    /**
     *  计量单位错
     */
    unitsError,
    /**
     * 标点错
     */
    punctuationError,
    /**
     * 语法错
     */
    grammarError,
    /**
     * 知识性差错
     */
    knowledgeableError,
    /**
     *  倾向性差错
     */
    tendentiousError,
    /**
     * 不⼀致
     */
    inconsistentError,
    /**
     * 格式错
     */
    formatError,
}
export type TProofreadItem = {
    /**
     *  id
     */
    id: int;
    /**
     * 用户ID
     */
    uid: string | number;
    /**
     * 文稿ID
     */
    fid: int;
    //md5加密字段(原始内容加密字段)
    key: string;
    /**
     * 原始内容
     */
    text: string;
    /**
     * 校验后的文本
     */
    newText: string;
    correctedContent: TCorrectedContent;
    /**
     * 检验时间
     */
    checkTime: int;
    offset: int;
    /**
     * 是否采纳 0默认 1忽略 2采纳 3未采纳
     */
    isAccept: ProofreadStateEnum;
    /**
     * 异常类型 0未知 1个人黑名单 2字错 3词错 4数字错 5标点错 6计量单位错 7语法错误 8知识性差错 9倾向性差错 10不一致 11格式错
     */
    type: ProofreadTypeEnum;
    /**
     * 创建时间
     */
    ctime: int;
    utime: int;
    dtime: int;
    action?: 'replace' | 'delete' | 'insert'
}

export type TNormalProofreadItem = {
    origin:string;
    text:string;
}
export type TOPS = {
    attributes?: {
        bold?: boolean;
        italic?: boolean;
        proofread?: {
            action: 'insert' | 'delete' | 'replace';
            id: number;
        },
        replace?: TProofreadItem;
        insert?: TNormalProofreadItem;
        delete?: TProofreadItem;
    },
    insert?: string;
}